#include "common.h"
#include "device_change.h"
#include "download.h"

#define BUFFER_SIZE 1024

// 检查并创建日志目录
static int create_devicename_directory_if_not_exists(const char *path) {
    char dir_path[256] = {0};
    const char *p = path;
    char *q = dir_path;
    
    // 提取目录部分
    while (*p != '\0') {
        *q = *p;
        if (*p == '/' && *(p+1) != '\0') {
            // 检查并创建每一级目录
            if (access(dir_path, F_OK) != 0) {
                if (mkdir(dir_path, 0755) != 0) {
                    fprintf(stderr, "Failed to create directory: %s\n", dir_path);
                    my_zlog_error("创建目录失败");
                    return -1;
                }
            }
        }
        p++;
        q++;
    }
    
    return 0;
}

// 复制文件函数
int copy_file(const char *source, const char *destination) {
    FILE *src_file = fopen(source, "rb");
    if (src_file == NULL) {
        my_zlog_warn("无法打开源文件");
        return -1;
    }

    FILE *dst_file = fopen(destination, "wb");
    if (dst_file == NULL) {
        my_zlog_warn("无法打开目标文件");
        fclose(src_file);
        return -1;
    }

    char buffer[BUFFER_SIZE];
    size_t bytes_read;
    while ((bytes_read = fread(buffer, 1, BUFFER_SIZE, src_file)) > 0) {
        if (fwrite(buffer, 1, bytes_read, dst_file) != bytes_read) {
            my_zlog_warn("写入目标文件时出错");
            fclose(src_file);
            fclose(dst_file);
            return -1;
        }
    }

    fclose(src_file);
    fclose(dst_file);
    return 0;
}

// 改名并移动文件函数
int rename_and_move_file(const char *source_file, const char *new_name, const char *destination_dir) {
    char destination_file[BUFFER_SIZE];
    // 构建目标文件的完整路径
    snprintf(destination_file, sizeof(destination_file), "%s/%s", destination_dir, new_name);

    // 尝试使用 rename 函数移动文件
    if (rename(source_file, destination_file) == 0) {
        my_zlog_debug("文件已成功改名并移动到 %s\n", destination_file);
        return 0;
    }

    // 如果 rename 失败，可能是跨文件系统，尝试复制文件
    if (errno == EXDEV) {
        if (copy_file(source_file, destination_file) == 0) {
            // 复制成功后删除原文件
            if (remove(source_file) == 0) {
                my_zlog_debug("文件已成功改名并移动到 %s\n", destination_file);
                return 0;
            } else {
                my_zlog_warn("删除原文件时出错");
                return -1;
            }
        } else {
            my_zlog_warn("复制文件时出错");
            return -1;
        }
    }

    my_zlog_warn("移动文件时出错");
    return -1;
}

int device_changename_back() {
    if(file_exists(DEVICE_NAME_FILE) == 0) {//先检测文件是否存在
        if(create_devicename_directory_if_not_exists(DEVICE_NAME_DIR) != 0) {
            my_zlog_error("创建文件目录失败");
            return -1;
        }

    char *DEVICE_NAME_NEW = NULL;

    rename_and_move_file(DEVICE_NAME_FILE,DEVICE_NAME_NEW,DEVICE_NAME_DIR);

    }
}